11/04/99 v1.1 liaor@umcc.ais.org      http://members.tripod.com/~liaor

-------------------------------------------
Inverse Discrete Cosine Transform functions
-------------------------------------------

     1. Contents of IDCT package
     2. Description
     3. Using the IDCT functions
     4. About the test results

-------------------------------------------
1) Contents
-------------------------------------------

I've collected a bunch of IDCT functions which I used in my MPEG2AVI
program.  I thought these functions might come in handy for other
programmers working on their own projects.
     
     8x8 IDCT functions
     ------------------
     1.  jrevdct.c - JPEG v4 reference iDCT (32-bit integer)
         came with the original ieee1180.tar.gz distribution
         http://www.mpeg.org
     
     
     2.  idct.c - MPEG-2 v1.2 reference Chen iDCT (32-bit integer)
         came with the MSSG MPEG2CODEC v1.2 (08/96)
         http://www.mpeg.org
     
     3.  idct_aan.c - Intel AAN iDCT (16-bit MMX)
         http://developer.intel.com/drg/mmx/appnotes - AppNote 528
     
     4.  Chen iDCT (16-bit MMX) - Khanh Nguyen-Phi's iDCT
         http://www.ece.ubc.ca/~knguyen/   ( k.p.nguyen@ieee.org )
     
     5.  idctmm32.c - Intel iDCT (32-bit MMX)
         http://developer.intel.com/vtunecbts/strmsimd - AppNote 922
     
     6.  idctmm32_transpose.c - Intel iDCT (32-bit MMX)
         transposing version of #5.  (slightly faster)
     
     A program-generated report for each iDCT is in \REPORTS
     
-------------------------------------------
2) Description
-------------------------------------------

          Well the functions are self-explanatory.  Each file contains
     a different implementation of the 8x8 inverse discrete cosine
     transform.
     
          For all functions, input is an array of sixty-four 16-bit
     signed-integers.  Output is in the same format, 16-bit signed-
     integers.  Some of the algorithms natively transpose the output;
     those that did so were modified to present the output matrix in
     non-transposed order.  (The MMX16-AAN and MMX32 iDCTs transpose
     the output.)  Each function compiles under Visual C++ 6.0 without
     modification.  (Other Win32 compilers may require external INC
     files for MMX support.)
     
          The algorithms represent different levels of accuracy and
     speed optimization.  On my test PC, the 16-bit MMX iDCTs (AAN and
     Chen) were the fastest, followed by the 32-bit MMX iDCT.
     
          *I* did not write these iDCT functions!  So don't email me
     about specific mathematical/numerical theory questions, please.
     I personally worked on the AAN iDCT and 32-bit MMX iDCT, so I can
     answer limited questions about COMPILING only!

-------------------------------------------
3)  Description
-------------------------------------------

  ----------------------------------------
  IDCT_AAN.C - 16-bit MMX AAN iDCT (Intel)

     Not IEEE 1180-1990 compliant
     
          In its original form, Intel's AP-528 iDCT (AAN) is pretty
     awful.  The iDCT requires input data to be left-shifted by 4
     bits, and the iDCT outputs the matrix in transposed order.
     Finally, the output is mean-shifted by approximately -0.5.  The
     mean-shift is sufficient to cause VISIBLE artifacts in MPEG-
     decoding engines, where an iDCT coefficients are applied as delta-
     values to inter-compressed frames.  The error propogates within
     MPEG sequences, and has the visual appearance of
     increasing/decreasing color/brightness shifts that cycle up and
     down.
     
     The modified AAN_IDCT automatically contains 2 noteworthy
     modifications :
     
     1)    In-place left-shift of input data. The left-shift
     eliminates the need to pre-shift the iDCT input coefficients.
     And the "in-place" aspect imposes minimal performance overhead (
     <10 cycles on Pentium/MMX.)  This generally faster than pre-
     scaling the data outside the AANiDCT function.
     
     2)   "mean-shift compensation", the DC-coefficient [iDCT(0,0)] is
     padded upward prior to transformation.  This reduces the mean-
     error to ~ -0.03 or so, enough to eliminate the color/brightness
     cycling artifacts described earlier.
     
          I added an output-transposer to idct_aan.c, to make the
     output compatible with the ieee1180 tester.  Since the transposer
     is "tacked on", it imposes a 20-30 cycle overhead.  If you do not
     need post-transpose, you can delete the code.

  ----------------------------------------------------
  IDCTCHEN.C - 16-bit MMX Chen1 iDCT (Khanh Nguyen-Phi)

     Not IEEE 1180-1990 compliant
     
          Although not IEEE compliant, this iDCT is just as fast as
     Intel's AAN, but more accurate.  Khanh's original Chen1 function
     contains raw assembly code that is not optimally scheduled.
          The modified Chen1_IDCT is (almost) optimally scheduled for
     Intel MMX CPUs.  K6-2/K7 CPUs will run the code slightly faster
     (thanks to AMD's faster pmul and dual pack/unpack.)  The hand-
     scheduling gives most benefit to the original Pentium/MMX.  More
     advanced CPUs like the K6-2 and Pentium-II benefit less.

  ---------------------------------------------------------
  IDCTMM32.C & IDCTMM32_TRANPOSE.C - 32-bit MMX iDCT (Intel)

     IEEE 1180-1990 compliant
     
          This IDCT uses 32-bit internal calculations more greater
     accuracy, at the expense of speed. Intel's application-note#922
     shows code-fragments for both MMX and SSE(SIMD.)  I pieced
     together a complete, working iDCT from the code-fragments.  I
     would love to do a PentiumIII (SSE) version, but I do not have a
     P3 CPU, so I wouldn't be able to test it.
     
          My implementation performs the iDCT operation in four steps:
     
          1) iDCT row-transformation (all 8 rows of input)
          2) transpose intermediate result matrix
          3) iDCT row-transformation (all 8 rows of intermediate
     result)
          4) transpose output matrix (if input is transposed, not
     necessary)
     
          Transposing the input-matrix would eliminated step (4).  In
     its current form, idctmm32.c is not fully optimized.  The code is
     organized for better readability, not performance.
     
     This package now contains two semi-optimized versions of the
     MMX32 iDCT:
          Version#1 is the "non-transposing" version.  Output is not
          transposed.
          Version#2 is the "transposing" version.  Output is
          transposed.
     
     #2 is slightly faster than #1, since the algorithm naturally
     transposes the output (which version #1 must "undo.")
     

   ----------------------------------------------
   Integer 32-bit iDCTs (JPEG and MPEG reference)

     IEEE 1180-1990 compliant
     
          These IDCTs are supplied as reference code.


-------------------------------------------
4)  Test results
-------------------------------------------

          This zip includes the original ieee1180.tar.gz package from
     www.mpeg.org.  A DOS batch-file (doieee.bat) has been added for
     Windows users, to automate the test procedure.  Currently, the
     test-program must be recompiled to include different iDCT
     functions.
          
     Test results for each iDCT are located in \REPORTS.

------------------
11/04/99 v1.1
      optimized idctmm32.c, added idctmm32_transpose.c
      modified ieee1180.c to issue an "emms" instruction after each
j_rev_dct() call
10/31/99 v1.0
     initial release, includes 3 MMX iDCT functions, and ieee1180 test-
reports
